'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _log = require('babel-runtime/core-js/math/log2');

var _log2 = _interopRequireDefault(_log);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var React = require('react');
var _ = require('../underscore_ext');
var styles = require('./DensityPlot.module.css');
var sc = require('science');

var bin = 20; //number of bins
var plotHeight = 35;
var plotWidth = 200;
var zoomFactor = 3;
var topColor = "#BAA0CF";
var bottomColor = "#79C6C5";

function calculateHeight(exp, max, min, plotHt, plotWidth, unit) {
	var minValue = unit === "tpm" ? (0, _log2.default)(0.001) : min;
	var pxWidth = (max - minValue) / plotWidth;
	var newExp = exp.filter(function (e) {
		return e > minValue;
	});
	var percentNonZero = newExp.length / exp.length;
	var kdePoints = newExp.length ? sc.stats.kde().sample(newExp)(_.range(minValue, max + pxWidth, pxWidth)) : [];
	var yHeights = kdePoints.map(function (kdep) {
		return kdep[1] * percentNonZero;
	});
	//  let binWidth = plotWidth / yHeights.length;
	var zPxWidth = 5; // draw zero at 5 px width
	var zeroWidth = pxWidth * zPxWidth;
	var zeroHeight = (1 - percentNonZero) / zeroWidth;
	//    let vscale = Math.max(zeroHeight, ...yHeights);
	//    //polyline points here
	//    let polylinePoints = [`0,${plotHt}`, ...yHeights.map((y, i) => `${i * binWidth},${(1 - y / vscale) * plotHt}`), `${plotWidth},${plotHt}`].join(' ');
	//
	// //   For debugging the scaling. The 'sum' should be 1: area under the curve
	// //   of a density plot should always be 1. We take the values from kde and scale
	// //   them by percentNonZero, then compute zeroHeight to perserve the correct area.
	// //
	// //   We then scale the y axis by the max y value, to ensure that the data is
	// //   not clipped (for a sharp spike), or drawn too small to see (for a uniform
	// //   distribution).
	// //
	// //   pxSum and zPxSum compute the pixel area for zero and non-zero points. They
	// //   should have the same ratio as zeroSum and nonZeroSum if we've scaled correctly.
	// //
	// //   var zeroSum = zeroWidth * zeroHeight;
	// //   var nonZeroSum = _.sum(kdePoints.map(([, y]) => y * pxWidth)) * percentNonZero;
	// //   var pxSum = _.sum(yHeights.map(y => y * plotHt * binWidth));
	// //   var zPxSum = zPxWidth * zeroHeight * plotHt;
	// //   console.log('sum', nonZeroSum + zeroSum, JSON.stringify({nonZeroSum, zeroSum}));
	// //   console.log(JSON.stringify({nz: pxSum / (pxSum + zPxSum), z: zPxSum / (pxSum + zPxSum)}));
	//    return {
	//      polylinePoints: polylinePoints,
	//      zeroWidth: zPxWidth,
	//      zeroHeight: zeroHeight / vscale * plotHt
	//    };
	return {
		yHeights: yHeights,
		zeroWidth: zPxWidth,
		zeroHeight: zeroHeight
	};
}

function calculateFrequency(exp, max, min) {
	var stepSize = (max - min) / bin,
	    freq = _.times(bin, _.constant(0));

	// We could do this immutably with reduce/assoc, but it would cause
	// many array copies. Could also do groupBy/count, but it seems excessive.
	exp.forEach(function (value) {
		freq[Math.floor((value - min) / stepSize)]++;
	});
	freq.forEach(function (value, index) {
		freq[index] = freq[index] / exp.length; //eslint-disable-line local/no-property-assignments
	});
	return freq;
}

function densitySvg(heights, height, totalWidth, vscale, A) {
	var yHeights = heights.yHeights,
	    zeroWidth = heights.zeroWidth,
	    zeroHeight = heights.zeroHeight,
	    binWidth = plotWidth / yHeights.length,
	    polylinePoints = ['20,' + height].concat((0, _toConsumableArray3.default)(yHeights.map(function (y, i) {
		return i * binWidth + 20 + ',' + (1 - y / vscale) * height;
	})), [plotWidth + 20 + ',' + height]).join(' '),
	    scaledZeroHeight = zeroHeight / vscale * height;

	return React.createElement(
		'svg',
		{ width: totalWidth, height: height },
		React.createElement('rect', { x: '0', y: height - scaledZeroHeight, width: zeroWidth, height: scaledZeroHeight, fill: A ? topColor : bottomColor }),
		React.createElement('polyline', { points: polylinePoints, fill: A ? topColor : bottomColor })
	);
}

var drawDensityPlot = function drawDensityPlot(min, max, totalWidth, unit, getNameZoom) {
	return function (studyA, studyB, nameAndZoom) {
		var height = nameAndZoom.zoom ? plotHeight * zoomFactor : plotHeight,
		    rowClass = nameAndZoom.zoom ? "densityPlot--row--zoom" : "densityPlot--row",
		    aHeight = calculateHeight(studyA.expA, max, min, height, plotWidth, unit),
		    bHeight = calculateHeight(studyB.expB, max, min, height, plotWidth, unit),
		    vscale = Math.max.apply(Math, [aHeight.zeroHeight, bHeight.zeroHeight].concat((0, _toConsumableArray3.default)(aHeight.yHeights), (0, _toConsumableArray3.default)(bHeight.yHeights)));
		return React.createElement(
			'div',
			{ key: nameAndZoom.name, className: styles[rowClass], style: { width: totalWidth + 'px' }, onClick: function onClick() {
					return getNameZoom(nameAndZoom.name);
				} },
			React.createElement('div', { className: styles["densityPlot--row--xAxis"], style: { width: plotWidth * 100 / totalWidth + '%', left: "20px" } }),
			React.createElement(
				'div',
				{ className: styles["densityPlot--row--studyA"], style: { width: totalWidth } },
				densitySvg(aHeight, height, totalWidth, vscale, true)
			),
			React.createElement(
				'div',
				{ className: styles["densityPlot--row--studyB"], style: { width: totalWidth } },
				densitySvg(bHeight, height, totalWidth, vscale, false)
			)
		);
	};
};

var histogramRowBin = function histogramRowBin(backgroundColor) {
	return function (freq) {
		return React.createElement('div', { className: styles["densityPlot--row--bin"],
			style: {
				height: freq * 100 + "%",
				width: 1 / bin * 100 + "%",
				backgroundColor: backgroundColor } });
	};
};

var drawHistogram = function drawHistogram(min, max) {
	return function (studyA, studyB) {
		var freqA = calculateFrequency(studyA.expA, max, min),
		    freqB = calculateFrequency(studyB.expB, max, min);
		return React.createElement(
			'div',
			{ className: styles["densityPlot--row"] },
			React.createElement('div', { className: styles["densityPlot--row--xAxis"] }),
			React.createElement(
				'div',
				{ className: styles["densityPlot--row--studyA"] },
				freqA.map(histogramRowBin(topColor))
			),
			React.createElement(
				'div',
				{ className: styles["densityPlot--row--studyB"] },
				freqB.map(histogramRowBin(bottomColor))
			)
		);
	};
};

var DensityPlot = function (_React$PureComponent) {
	(0, _inherits3.default)(DensityPlot, _React$PureComponent);

	function DensityPlot() {
		(0, _classCallCheck3.default)(this, DensityPlot);
		return (0, _possibleConstructorReturn3.default)(this, (DensityPlot.__proto__ || (0, _getPrototypeOf2.default)(DensityPlot)).apply(this, arguments));
	}

	(0, _createClass3.default)(DensityPlot, [{
		key: 'render',
		value: function render() {
			var _props = this.props,
			    unit = _props.unit,
			    getNameZoom = _props.getNameZoom,
			    type = _props.type,
			    totalWidth = plotWidth + 20,
			    data = this.props.data ? this.props.data : null,
			    max = _.max(_.flatten(_.pluck(data.studyA, "expA").concat(_.pluck(data.studyB, "expB")))),
			    min = _.min(_.flatten(_.pluck(data.studyA, "expA").concat(_.pluck(data.studyB, "expB")))),
			    drawPlot = (type === 'density' ? drawDensityPlot : drawHistogram)(min, max, totalWidth, unit, getNameZoom),
			    rows = _.mmap(data.studyA, data.studyB, data.nameAndZoom, drawPlot);

			return React.createElement(
				'div',
				{ className: styles.densityPlot },
				rows
			);
		}
	}]);
	return DensityPlot;
}(React.PureComponent);

module.exports = { DensityPlot: DensityPlot, bottomColor: bottomColor, topColor: topColor, plotWidth: plotWidth };